#include <stdio.h>
#include <stdlib.h>

/* the board */
struct board {
    int w, h;
    char* grid;
};

/* the four directions are defined as binary numbers so we can combine them */
#define N 1
#define S 2
#define E 4
#define W 8

/* return a char giving a hex digit of such a number */
char hex(int dir) {
    if (dir < 10) {
        return dir + '0';
    } else {
        return (dir - 10) + 'a';
    }
}

/* get a number out of a hex char */
int val(char c) {
    if (c >= '0' && c <= '9') {
        return c - '0';
    } else {
        return (c - 'a') + 10;
    }
}

/* return whether we can walk along a path in some direction */
int cango(char space, int dir) {
    /* simple cases */
    switch (space) {
        case ' ': return 1;
        case '#': return 0;
        case 'S': return 1;
        case 'X': return 0;
        case 'H': return 1;
        case 'B': return 0;
        case '/': return 0;
        case '\\': return 0;
    }

    /* it's one of the dir thingies, check if it has that bit */
    int v = val(space);

    /* if that bit is set, then we CANNOT go this way */
    if (v & dir) {
        return 0;
    }

    /* else we can :) */
    return 1;
}


/* return the cell of a given board, or WALL if out of bounds */
char cell(struct board* b, int row, int col) {
    if (row < 0 || row >= b->h || col < 0 || col > b->w) {
        return '#';
    }
    return b->grid[row * b->w + col];
}


/* set a cell on the board */
void set_cell(struct board* b, int row, int col, char new) {
    b->grid[row * b->w + col] = new;
}


/* read a baord from stdin */
struct board* read_board() {
    struct board* b = malloc(sizeof (struct board));
    scanf("%d", &b->w);
    scanf("%d", &b->h);
    scanf("\n");

    b->grid = malloc(b->w * b->h * sizeof(char));
    int row, col;
    for (row = 0; row < b->h; row++) {
        for (col = 0; col < b->w; col++) {
            scanf("%c", b->grid + row*b->w + col);
        }
        scanf("\n");
    }
    
    return b;
}


/* dump a board for debugging */
void dump_board(struct board* b) {
    int row, col;
    for (row = 0; row < b->h; row++) {
        for (col = 0; col < b->w; col++) {
            printf("%c", cell(b, row, col));
        }
        printf("\n");
    }
}


/* trace the basilisk's line of site out from its initial position along any
 * mirrors, mark cells with a view of the basilisk as N (view to north) E (view
 * to east) and so on */
void raycast(struct board* b, int row, int col, char dir) {
    /* find the next cell we are considering */
    int nrow = row, ncol = col;
    switch (dir) {
        case N: nrow--; break;
        case S: nrow++; break;
        case E: ncol++; break;
        case W: ncol--; break;
    }

    /* check what kind of cell this is */
    char c = cell(b, nrow, ncol);
    switch (c) {
        case ' ':
            /* keep on trucking */
            set_cell(b, nrow, ncol, hex(dir));
            raycast(b, nrow, ncol, dir);
            break;

        case '/':
            /* we have a reflection - we don't recurse into the n cell, but
             * instead a cell in the direction of the mirror! */
            switch (dir) {
                case N: set_cell(b, nrow, ncol + 1, hex(E)); raycast(b, nrow, ncol + 1, E); break;
                case S: set_cell(b, nrow, ncol - 1, hex(W)); raycast(b, nrow, ncol - 1, W); break;
                case E: set_cell(b, nrow - 1, ncol, hex(N)); raycast(b, nrow - 1, ncol, N); break;
                case W: set_cell(b, nrow + 1, ncol, hex(S)); raycast(b, nrow + 1, ncol, S); break;
            }
            break;

        case '\\':
            /* we have the other kind of reflection! */
            switch (dir) {
                case N: set_cell(b, nrow, ncol - 1, hex(W)); raycast(b, nrow, ncol - 1, W); break;
                case S: set_cell(b, nrow, ncol + 1, hex(E)); raycast(b, nrow, ncol + 1, E); break;
                case E: set_cell(b, nrow + 1, ncol, hex(S)); raycast(b, nrow + 1, ncol, S); break;
                case W: set_cell(b, nrow - 1, ncol, hex(N)); raycast(b, nrow - 1, ncol, N); break;
            }
            break;

        default:
            if ((c >= '0' && c <= '9') || (c >= 'a' && c <= 'f')) {
                int value = val(c);
                set_cell(b, nrow, ncol, hex(value | dir));
                raycast(b, nrow, ncol, dir);
            }
    }
}


/* call the raycast function above starting at the basilisk position in all four
 * directions outward */
void raycast_all(struct board* b) {
    /* start by finding the basilisk */
    int row, col;
    for (row = 0; row < b->h; row++) {
        for (col = 0; col < b->w; col++) {
            if (cell(b, row, col) == 'B') {
                raycast(b, row, col, N);
                raycast(b, row, col, E);
                raycast(b, row, col, S);
                raycast(b, row, col, W);
            }
        }
    }
}

/* free memory allocated to board */
void free_board(struct board* b) {
    free(b->grid);
    free(b);
}

/* do dfs at a given point */
int dfsp(struct board* b, int hrow, int hcol, int srow, int scol) {
    /* check if we're done */
    if (hrow == srow && hcol == scol) {
        return 1;
    }

    /* mark this spot as seen */
    set_cell(b, hrow, hcol, 'X');

    /* get the chars in each direction */
    char left = cell(b, hrow, hcol - 1);
    char right = cell(b, hrow, hcol + 1);
    char up = cell(b, hrow - 1, hcol);
    char down = cell(b, hrow + 1, hcol);

    /* if we can go left, search there */
    if (cango(left, E))
        if (dfsp(b, hrow, hcol - 1, srow, scol))
            return 1;

    /* if we can go right, search there */
    if (cango(right, W))
        if (dfsp(b, hrow, hcol + 1, srow, scol))
            return 1;

    /* if we can go up, search there */
    if (cango(up, S))
        if (dfsp(b, hrow - 1, hcol, srow, scol))
            return 1;

    /* if we can go down, search there */
    if (cango(down, N))
        if (dfsp(b, hrow + 1, hcol, srow, scol))
            return 1;

    /* if we get all the way down here, we failed :( */
    return 0;
}


/* do a dfs from harry to sword */
int dfs(struct board* b) {
    /* start by finding harry and sword */
    int row, col, harry_row, harry_col, target_row, target_col;
    for (row = 0; row < b->h; row++) {
        for (col = 0; col < b->w; col++) {
            if (cell(b, row, col) == 'H') {
                harry_row = row;
                harry_col = col;
            }
            if (cell(b, row, col) == 'S') {
                target_row = row;
                target_col = col;
            }
        }
    }

    /* start the recursive process here */
    return dfsp(b, harry_row, harry_col, target_row, target_col);
}

/* the whole thang */
int main() {
    struct board* b = read_board();
    raycast_all(b);

    /* check if harry can get to the sword */
    int can = dfs(b);

    printf("Harry can %sreach the sword!\n", can ? "" : "not ");

    free_board(b);
    return 0;
}

