/* this solution uses a polynomial time, graph algorithm to solve
 * the problem, and is thus quite fast, it is trickier though! */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_DOORS 100
#define TRANS_TIME 5.0

#define INF 9999999

/* the graph is an adjacency matrix
 *
 * if there are three doors, the structure is:
 *
 * 0 ------2-4------6-8------10-12------14
 * |       | |      | |       |  |       |
 * 1-------3-5------7-9------11-13------15
 *
 * The top row is "human mode", the bottom row is "dog mode".  The cost from
 * the top to bottom or bottom to top is the 5 second transformation time.
 *
 * The paths from 0-2, 1-3, 4-6, 5-7, etc. are the costs of traversing that stretch
 * of the hallway.  The cost is length/4 for human mode and length/10 for dog.
 *
 * The paths from 2-4, 3-5, 6-8, 7-8, etc. are the costs of opening the door.  For
 * the top row, it's 1 second, for the bottom it's 8.
 *
 * We start at 0 and need to get to 14 or 15.
 */

struct {
    /* total length of the path */
    double total_distance;

    /* number of doors on the path */
    int ndoors;

    /* number of nodes in the graph */
    int nodes;

    /* the cost between any two nodes in the graph */
    double** costs;
} graph;

double dijkstra(int* trans) {
    int start = 0, i;

    /* keep track of best path to each node so far */
    double* tentative = malloc(graph.nodes * sizeof(double));
    for (i = 0; i < graph.nodes; i++) {
        tentative[i] = INF;
    }

    /* cost to 0 is nothing */
    tentative[start] = 0;

    /* keep track of where we have been */
    int* visited = malloc(graph.nodes * sizeof(int));
    for (i = 0; i < graph.nodes; i++) {
        visited[i] = 0;
    }

    /* keep track of transformations taken */
    int* transformations = malloc(graph.nodes * sizeof(int));
    for (i = 0; i < graph.nodes; i++) {
        transformations[i] = 0;
    }

    /* start at the beginning node */
    int current = start;
    while (current != -1) {
        /* for each neighbor */
        for (i = 0; i < graph.nodes; i++) {
            /* if there is an edge here */
            if (graph.costs[current][i] < INF) {
                /* calculate distance from start to i via current */
                double distance = tentative[current] + graph.costs[current][i];



                /* if this is less than current tentative distance, replace it */
                if (distance < tentative[i]) {
                    tentative[i] = distance;
                    visited[i] = 0;
                    /* this is a transformation step (even to odd or odd to even) */
                    if (((current + i) % 2) == 1) {
                        transformations[i] = transformations[current] + 1;
                    } else {
                        transformations[i] = transformations[current];
                    }
                }
            }
        }

        /* mark current as visited */
        visited[current] = 1;

        /* set current node to unvisited one with smallest cost */
        double min = INF;
        current = -1;
        for (i = 0; i < graph.nodes; i++) {
            if ((!visited[i]) && (tentative[i] <= min)) {
                min = tentative[i];
                current = i;
            }
        }
    }

    /* we can reach either of these two nodes (end as man or end as dog) */
    int end1 = graph.nodes - 1;
    int end2 = graph.nodes - 2;
    double cost;
    if (tentative[end1] < tentative[end2]) {
        *trans = transformations[end1];
        cost = tentative[end1];
    } else {
    *trans = 
        *trans = transformations[end2];
        cost = tentative[end2];
    }

    free(visited);
    free(tentative);
    free(transformations);

    return cost;
}


int main() {
    int i, j;

    /* read stats */
    scanf("%lf", &graph.total_distance);
    scanf("%d", &graph.ndoors);

    /* read doors into temp array */
    double doors[MAX_DOORS];
    for (i = 0; i < graph.ndoors; i++) {
        scanf("%lf", doors + i);
    }

    /* alloc the graph */
    int n = graph.nodes = (graph.ndoors*2 + 2) * 2;
    graph.costs = malloc(n * sizeof(double*));
    for (i = 0; i < n; i++) {
        graph.costs[i] = malloc(n * (sizeof(double)));
    }

    /* init to all very expensive */
    for (i = 0; i < n; i++) {
        for (j = 0; j < n; j++) {
            graph.costs[i][j] = INF;
        }
    }

    /* build costs along running paths */
    double a = 0;
    double b = doors[0];
    int doori = 1;
    for (i = 0; i <= graph.ndoors; i++) {
        graph.costs[i * 4][i * 4 + 2] = (b - a) / 4.0;
        graph.costs[i * 4 + 1][i * 4 + 2 + 1] = (b - a) / 10.0;

        a = b;
        if (doori < graph.ndoors) {
            b = doors[doori];
            doori++;
        } else {
            b = graph.total_distance;
        }
    }


    /* build door opening paths */
    int indexer = 2;
    for (i = 0; i < graph.ndoors; i++) {
        graph.costs[indexer][indexer + 2] = 1;
        graph.costs[indexer + 1][indexer + 3] = 8;
        indexer += 4;
    }

    /* build costs from human/wolf */
    for (i = 0; i < n/2; i++) {
        graph.costs[i * 2][i * 2 + 1] = 5;
        graph.costs[i * 2 + 1][i * 2] = 5;
    }


    /* run dijsktra */
    int tran;
    double cost = dijkstra(&tran);


    for (i = 0; i < n; i++) {
        free(graph.costs[i]);
    }
    free(graph.costs);

    printf("Sirius could travel this distance in %.3lf seconds, "
           "transforming %d time%s.\n", cost, tran, tran == 1 ? "" : "s");

    return 0;
}

